# Core Data

Core Data is a [data module](https://github.com/WordPress/gutenberg/tree/HEAD/packages/data/README.md) intended to simplify access to and manipulation of core WordPress entities. It registers its own store and provides a number of selectors which resolve data from the WordPress REST API automatically, along with dispatching action creators to manipulate data. Core data is shipped with [`TypeScript definitions for WordPress data types`](https://github.com/WordPress/gutenberg/tree/HEAD/packages/core-data/src/types/README.md).

Used in combination with features of the data module such as [`subscribe`](https://github.com/WordPress/gutenberg/tree/HEAD/packages/data/README.md#subscribe-function) or [higher-order components](https://github.com/WordPress/gutenberg/tree/HEAD/packages/data/README.md#higher-order-components), it enables a developer to easily add data into the logic and display of their plugin.

## Installation

Install the module

```bash
npm install @wordpress/core-data --save
```

_This package assumes that your code will run in an **ES2015+** environment. If you're using an environment that has limited or no support for such language features and APIs, you should include [the polyfill shipped in `@wordpress/babel-preset-default`](https://github.com/WordPress/gutenberg/tree/HEAD/packages/babel-preset-default#polyfill) in your code._

## Example

Below is an example of a component which simply renders a list of authors:

```jsx
const { useSelect } = wp.data;

function MyAuthorsListBase() {
	const authors = useSelect( ( select ) => {
		return select( 'core' ).getUsers( { who: 'authors' } );
	}, [] );

	if ( ! authors ) {
		return null;
	}

	return (
		<ul>
			{ authors.map( ( author ) => (
				<li key={ author.id }>{ author.name }</li>
			) ) }
		</ul>
	);
}
```

## Actions

The following set of dispatching action creators are available on the object returned by `wp.data.dispatch( 'core' )`:

<!-- START TOKEN(Autogenerated actions|src/actions.js) -->

### addEntities

Returns an action object used in adding new entities.

_Parameters_

-   _entities_ `Array`: Entities received.

_Returns_

-   `Object`: Action object.

### deleteEntityRecord

Action triggered to delete an entity record.

_Parameters_

-   _kind_ `string`: Kind of the deleted entity.
-   _name_ `string`: Name of the deleted entity.
-   _recordId_ `string`: Record ID of the deleted entity.
-   _query_ `?Object`: Special query parameters for the DELETE API call.
-   _options_ `[Object]`: Delete options.
-   _options.\_\_unstableFetch_ `[Function]`: Internal use only. Function to call instead of `apiFetch()`. Must return a promise.

### editEntityRecord

Returns an action object that triggers an
edit to an entity record.

_Parameters_

-   _kind_ `string`: Kind of the edited entity record.
-   _name_ `string`: Name of the edited entity record.
-   _recordId_ `number`: Record ID of the edited entity record.
-   _edits_ `Object`: The edits.
-   _options_ `Object`: Options for the edit.
-   _options.undoIgnore_ `boolean`: Whether to ignore the edit in undo history or not.

_Returns_

-   `Object`: Action object.

### receiveAutosaves

Returns an action object used in signalling that the autosaves for a
post have been received.

_Parameters_

-   _postId_ `number`: The id of the post that is parent to the autosave.
-   _autosaves_ `Array|Object`: An array of autosaves or singular autosave object.

_Returns_

-   `Object`: Action object.

### receiveCurrentTheme

Returns an action object used in signalling that the current theme has been received.

_Parameters_

-   _currentTheme_ `Object`: The current theme.

_Returns_

-   `Object`: Action object.

### receiveCurrentUser

Returns an action used in signalling that the current user has been received.

_Parameters_

-   _currentUser_ `Object`: Current user object.

_Returns_

-   `Object`: Action object.

### receiveEmbedPreview

Returns an action object used in signalling that the preview data for
a given URl has been received.

_Parameters_

-   _url_ `string`: URL to preview the embed for.
-   _preview_ `*`: Preview data.

_Returns_

-   `Object`: Action object.

### receiveEntityRecords

Returns an action object used in signalling that entity records have been received.

_Parameters_

-   _kind_ `string`: Kind of the received entity.
-   _name_ `string`: Name of the received entity.
-   _records_ `Array|Object`: Records received.
-   _query_ `?Object`: Query Object.
-   _invalidateCache_ `?boolean`: Should invalidate query caches.
-   _edits_ `?Object`: Edits to reset.

_Returns_

-   `Object`: Action object.

### receiveThemeSupports

> **Deprecated** since WP 5.9, this is not useful anymore, use the selector direclty.

Returns an action object used in signalling that the index has been received.

_Returns_

-   `Object`: Action object.

### receiveUploadPermissions

> **Deprecated** since WP 5.9, use receiveUserPermission instead.

Returns an action object used in signalling that Upload permissions have been received.

_Parameters_

-   _hasUploadPermissions_ `boolean`: Does the user have permission to upload files?

_Returns_

-   `Object`: Action object.

### receiveUserPermission

Returns an action object used in signalling that the current user has
permission to perform an action on a REST resource.

_Parameters_

-   _key_ `string`: A key that represents the action and REST resource.
-   _isAllowed_ `boolean`: Whether or not the user can perform the action.

_Returns_

-   `Object`: Action object.

### receiveUserQuery

Returns an action object used in signalling that authors have been received.

_Parameters_

-   _queryID_ `string`: Query ID.
-   _users_ `Array|Object`: Users received.

_Returns_

-   `Object`: Action object.

### redo

Action triggered to redo the last undoed
edit to an entity record, if any.

_Returns_

-   `undefined`:

### saveEditedEntityRecord

Action triggered to save an entity record's edits.

_Parameters_

-   _kind_ `string`: Kind of the entity.
-   _name_ `string`: Name of the entity.
-   _recordId_ `Object`: ID of the record.
-   _options_ `Object`: Saving options.

### saveEntityRecord

Action triggered to save an entity record.

_Parameters_

-   _kind_ `string`: Kind of the received entity.
-   _name_ `string`: Name of the received entity.
-   _record_ `Object`: Record to be saved.
-   _options_ `Object`: Saving options.
-   _options.isAutosave_ `[boolean]`: Whether this is an autosave.
-   _options.\_\_unstableFetch_ `[Function]`: Internal use only. Function to call instead of `apiFetch()`. Must return a promise.

### undo

Action triggered to undo the last edit to
an entity record, if any.

_Returns_

-   `undefined`:

<!-- END TOKEN(Autogenerated actions|src/actions.js) -->

## Selectors

The following selectors are available on the object returned by `wp.data.select( 'core' )`:

<!-- START TOKEN(Autogenerated selectors|src/selectors.js) -->

### canUser

Returns whether the current user can perform the given action on the given
REST resource.

Calling this may trigger an OPTIONS request to the REST API via the
`canUser()` resolver.

<https://developer.wordpress.org/rest-api/reference/>

_Parameters_

-   _state_ `Object`: Data state.
-   _action_ `string`: Action to check. One of: 'create', 'read', 'update', 'delete'.
-   _resource_ `string`: REST resource to check, e.g. 'media' or 'posts'.
-   _id_ `string=`: Optional ID of the rest resource to check.

_Returns_

-   `boolean|undefined`: Whether or not the user can perform the action, or `undefined` if the OPTIONS request is still being made.

### canUserEditEntityRecord

Returns whether the current user can edit the given entity.

Calling this may trigger an OPTIONS request to the REST API via the
`canUser()` resolver.

<https://developer.wordpress.org/rest-api/reference/>

_Parameters_

-   _state_ `Object`: Data state.
-   _kind_ `string`: Entity kind.
-   _name_ `string`: Entity name.
-   _recordId_ `string`: Record's id.

_Returns_

-   `boolean|undefined`: Whether or not the user can edit, or `undefined` if the OPTIONS request is still being made.

### getAuthors

> **Deprecated** since 11.3. Callers should use `select( 'core' ).getUsers({ who: 'authors' })` instead.

Returns all available authors.

_Parameters_

-   _state_ `Object`: Data state.
-   _query_ `Object|undefined`: Optional object of query parameters to include with request.

_Returns_

-   `Array`: Authors list.

### getAutosave

Returns the autosave for the post and author.

_Parameters_

-   _state_ `Object`: State tree.
-   _postType_ `string`: The type of the parent post.
-   _postId_ `number`: The id of the parent post.
-   _authorId_ `number`: The id of the author.

_Returns_

-   `?Object`: The autosave for the post and author.

### getAutosaves

Returns the latest autosaves for the post.

May return multiple autosaves since the backend stores one autosave per
author for each post.

_Parameters_

-   _state_ `Object`: State tree.
-   _postType_ `string`: The type of the parent post.
-   _postId_ `number`: The id of the parent post.

_Returns_

-   `?Array`: An array of autosaves for the post, or undefined if there is none.

### getCurrentTheme

Return the current theme.

_Parameters_

-   _state_ `Object`: Data state.

_Returns_

-   `Object`: The current theme.

### getCurrentUser

Returns the current user.

_Parameters_

-   _state_ `Object`: Data state.

_Returns_

-   `Object`: Current user object.

### getEditedEntityRecord

Returns the specified entity record, merged with its edits.

_Parameters_

-   _state_ `Object`: State tree.
-   _kind_ `string`: Entity kind.
-   _name_ `string`: Entity name.
-   _recordId_ `number`: Record ID.

_Returns_

-   `Object?`: The entity record, merged with its edits.

### getEmbedPreview

Returns the embed preview for the given URL.

_Parameters_

-   _state_ `Object`: Data state.
-   _url_ `string`: Embedded URL.

_Returns_

-   `*`: Undefined if the preview has not been fetched, otherwise, the preview fetched from the embed preview API.

### getEntitiesByKind

Returns whether the entities for the give kind are loaded.

_Parameters_

-   _state_ `Object`: Data state.
-   _kind_ `string`: Entity kind.

_Returns_

-   `Array<Object>`: Array of entities with config matching kind.

### getEntity

Returns the entity object given its kind and name.

_Parameters_

-   _state_ `Object`: Data state.
-   _kind_ `string`: Entity kind.
-   _name_ `string`: Entity name.

_Returns_

-   `Object`: Entity

### getEntityRecord

Returns the Entity's record object by key. Returns `null` if the value is not
yet received, undefined if the value entity is known to not exist, or the
entity object if it exists and is received.

_Parameters_

-   _state_ `Object`: State tree
-   _kind_ `string`: Entity kind.
-   _name_ `string`: Entity name.
-   _key_ `number`: Record's key
-   _query_ `?Object`: Optional query.

_Returns_

-   `Object?`: Record.

### getEntityRecordEdits

Returns the specified entity record's edits.

_Parameters_

-   _state_ `Object`: State tree.
-   _kind_ `string`: Entity kind.
-   _name_ `string`: Entity name.
-   _recordId_ `number`: Record ID.

_Returns_

-   `Object?`: The entity record's edits.

### getEntityRecordNonTransientEdits

Returns the specified entity record's non transient edits.

Transient edits don't create an undo level, and
are not considered for change detection.
They are defined in the entity's config.

_Parameters_

-   _state_ `Object`: State tree.
-   _kind_ `string`: Entity kind.
-   _name_ `string`: Entity name.
-   _recordId_ `number`: Record ID.

_Returns_

-   `Object?`: The entity record's non transient edits.

### getEntityRecords

Returns the Entity's records.

_Parameters_

-   _state_ `Object`: State tree
-   _kind_ `string`: Entity kind.
-   _name_ `string`: Entity name.
-   _query_ `?Object`: Optional terms query.

_Returns_

-   `?Array`: Records.

### getLastEntityDeleteError

Returns the specified entity record's last delete error.

_Parameters_

-   _state_ `Object`: State tree.
-   _kind_ `string`: Entity kind.
-   _name_ `string`: Entity name.
-   _recordId_ `number`: Record ID.

_Returns_

-   `Object?`: The entity record's save error.

### getLastEntitySaveError

Returns the specified entity record's last save error.

_Parameters_

-   _state_ `Object`: State tree.
-   _kind_ `string`: Entity kind.
-   _name_ `string`: Entity name.
-   _recordId_ `number`: Record ID.

_Returns_

-   `Object?`: The entity record's save error.

### getRawEntityRecord

Returns the entity's record object by key,
with its attributes mapped to their raw values.

_Parameters_

-   _state_ `Object`: State tree.
-   _kind_ `string`: Entity kind.
-   _name_ `string`: Entity name.
-   _key_ `number`: Record's key.

_Returns_

-   `Object?`: Object with the entity's raw attributes.

### getRedoEdit

Returns the next edit from the current undo offset
for the entity records edits history, if any.

_Parameters_

-   _state_ `Object`: State tree.

_Returns_

-   `Object?`: The edit.

### getReferenceByDistinctEdits

Returns a new reference when edited values have changed. This is useful in
inferring where an edit has been made between states by comparison of the
return values using strict equality.

_Usage_

    const hasEditOccurred = (
       getReferenceByDistinctEdits( beforeState ) !==
       getReferenceByDistinctEdits( afterState )
    );

_Parameters_

-   _state_ `Object`: Editor state.

_Returns_

-   `*`: A value whose reference will change only when an edit occurs.

### getThemeSupports

Return theme supports data in the index.

_Parameters_

-   _state_ `Object`: Data state.

_Returns_

-   `*`: Index data.

### getUndoEdit

Returns the previous edit from the current undo offset
for the entity records edits history, if any.

_Parameters_

-   _state_ `Object`: State tree.

_Returns_

-   `Object?`: The edit.

### getUserQueryResults

Returns all the users returned by a query ID.

_Parameters_

-   _state_ `Object`: Data state.
-   _queryID_ `string`: Query ID.

_Returns_

-   `Array`: Users list.

### hasEditsForEntityRecord

Returns true if the specified entity record has edits,
and false otherwise.

_Parameters_

-   _state_ `Object`: State tree.
-   _kind_ `string`: Entity kind.
-   _name_ `string`: Entity name.
-   _recordId_ `number`: Record ID.

_Returns_

-   `boolean`: Whether the entity record has edits or not.

### hasEntityRecords

Returns true if records have been received for the given set of parameters,
or false otherwise.

_Parameters_

-   _state_ `Object`: State tree
-   _kind_ `string`: Entity kind.
-   _name_ `string`: Entity name.
-   _query_ `?Object`: Optional terms query.

_Returns_

-   `boolean`: Whether entity records have been received.

### hasFetchedAutosaves

Returns true if the REST request for autosaves has completed.

_Parameters_

-   _state_ `Object`: State tree.
-   _postType_ `string`: The type of the parent post.
-   _postId_ `number`: The id of the parent post.

_Returns_

-   `boolean`: True if the REST request was completed. False otherwise.

### hasRedo

Returns true if there is a next edit from the current undo offset
for the entity records edits history, and false otherwise.

_Parameters_

-   _state_ `Object`: State tree.

_Returns_

-   `boolean`: Whether there is a next edit or not.

### hasUndo

Returns true if there is a previous edit from the current undo offset
for the entity records edits history, and false otherwise.

_Parameters_

-   _state_ `Object`: State tree.

_Returns_

-   `boolean`: Whether there is a previous edit or not.

### isAutosavingEntityRecord

Returns true if the specified entity record is autosaving, and false otherwise.

_Parameters_

-   _state_ `Object`: State tree.
-   _kind_ `string`: Entity kind.
-   _name_ `string`: Entity name.
-   _recordId_ `number`: Record ID.

_Returns_

-   `boolean`: Whether the entity record is autosaving or not.

### isDeletingEntityRecord

Returns true if the specified entity record is deleting, and false otherwise.

_Parameters_

-   _state_ `Object`: State tree.
-   _kind_ `string`: Entity kind.
-   _name_ `string`: Entity name.
-   _recordId_ `number`: Record ID.

_Returns_

-   `boolean`: Whether the entity record is deleting or not.

### isPreviewEmbedFallback

Determines if the returned preview is an oEmbed link fallback.

WordPress can be configured to return a simple link to a URL if it is not embeddable.
We need to be able to determine if a URL is embeddable or not, based on what we
get back from the oEmbed preview API.

_Parameters_

-   _state_ `Object`: Data state.
-   _url_ `string`: Embedded URL.

_Returns_

-   `boolean`: Is the preview for the URL an oEmbed link fallback.

### isRequestingEmbedPreview

Returns true if a request is in progress for embed preview data, or false
otherwise.

_Parameters_

-   _state_ `Object`: Data state.
-   _url_ `string`: URL the preview would be for.

_Returns_

-   `boolean`: Whether a request is in progress for an embed preview.

### isSavingEntityRecord

Returns true if the specified entity record is saving, and false otherwise.

_Parameters_

-   _state_ `Object`: State tree.
-   _kind_ `string`: Entity kind.
-   _name_ `string`: Entity name.
-   _recordId_ `number`: Record ID.

_Returns_

-   `boolean`: Whether the entity record is saving or not.

<!-- END TOKEN(Autogenerated selectors|src/selectors.js) -->

## Contributing to this package

This is an individual package that's part of the Gutenberg project. The project is organized as a monorepo. It's made up of multiple self-contained software packages, each with a specific purpose. The packages in this monorepo are published to [npm](https://www.npmjs.com/) and used by [WordPress](https://make.wordpress.org/core/) as well as other software projects.

To find out more about contributing to this package or Gutenberg as a whole, please read the project's main [contributor guide](https://github.com/WordPress/gutenberg/tree/HEAD/CONTRIBUTING.md).

<br /><br /><p align="center"><img src="https://s.w.org/style/images/codeispoetry.png?1" alt="Code is Poetry." /></p>
